/*-------------------------------------------------------------------------
 *
 * pmevent.c
 *	  routines for signaling the postmaster from users
 *
 *
 * Portions Copyright (c) 1996-2010, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * IDENTIFICATION
 *	  $PostgreSQL: pgsql/src/backend/storage/ipc/pmevent.c, Exp $
 *
 *-------------------------------------------------------------------------
 */
#include "postgres.h"

#include <sys/stat.h>
#include <unistd.h>

#include "storage/fd.h"
#include "storage/pmevent.h"


/*
 * CheckPostmasterEvent - check to see if a particular reason has been
 * signaled, and remove the event file.  Should be called by postmaster
 * after receiving SIGUSR1.
 */
bool
CheckPostmasterEvent(const char *eventfile)
{
	struct stat stat_buf;
	char		path[MAXPGPATH];

	PMEventFilePath(path, PMEVENTDIR, eventfile);
	if (stat(path, &stat_buf) == 0)
	{
		/*
		 * Since we are in a signal handler, it's not safe
		 * to elog. We silently ignore the error of unlink.
		 */
		unlink(path);
		return true;
	}
	return false;
}

/*
 * Remove the event files during startup.
 */
void
CleanupEventFiles(void)
{
	DIR		   *dir;
	struct dirent *de;
	char		path[MAXPGPATH];

	dir = AllocateDir(PMEVENTDIR);
	if (dir == NULL)
		ereport(FATAL,
				(errcode_for_file_access(),
				 errmsg("could not open postmaster event directory \"%s\": %m",
						PMEVENTDIR)));

	while ((de = ReadDir(dir, PMEVENTDIR)) != NULL)
	{
		if (strcmp(de->d_name, ".") == 0 ||
			strcmp(de->d_name, "..") == 0)
			continue;

		PMEventFilePath(path, PMEVENTDIR, de->d_name);
		if (unlink(path) != 0)
			ereport(FATAL,
					(errcode_for_file_access(),
					 errmsg("could not remove postmaster event file \"%s\": %m",
							path)));
	}

	FreeDir(dir);
}
